@@ -4,11 +4,16 @@ __license__ = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, time, sys, shutil, json, mimetypes
-from pprint import pprint
+import json
+import mimetypes
+import os
+import shutil
+import sys
+import time
 from argparse import ArgumentParser, FileType
-from subprocess import check_call
 from collections import OrderedDict
+from pprint import pprint
+from subprocess import check_call
 
 
 class ReadFileWithProgressReporting:  # {{{
@@ -51,20 +56,19 @@ class ReadFileWithProgressReporting:  # {{{
         eta = int((self._total - self.tell()) / bit_rate) + 1
         eta_m, eta_s = eta / 60, eta % 60
         sys.stdout.write(
-            '  %.1f%%   %.1f/%.1fMB %.1f KB/sec    %d minutes, %d seconds left' %
-            (frac * 100, mb_pos, mb_tot, kb_rate, eta_m, eta_s)
+            f'  {frac * 100:.1f}%   {mb_pos:.1f}/{mb_tot:.1f}MB {kb_rate:.1f} KB/sec    {eta_m} minutes, {eta_s} seconds left'
+
         )
         sys.stdout.write('\x1b[u')
         if self.tell() >= self._total:
             sys.stdout.write('\n')
             t = int(time.time() - self.start_time) + 1
             print(
-                'Upload took %d minutes and %d seconds at %.1f KB/sec' %
-                (t / 60, t % 60, kb_rate)
+                f'Upload took {t/60} minutes and {t % 60} seconds at {kb_rate:.1f} KB/sec'
+
             )
         sys.stdout.flush()
 
-
 # }}}
 
 
@@ -88,7 +92,6 @@ class Base:  # {{{
         print('_' * 50)
         sys.stdout.flush()
 
-
 # }}}
 
 
@@ -111,8 +114,7 @@ class SourceForge(Base):  # {{{
                 try:
                     check_call([
                         'rsync', '-h', '-zz', '--progress', '-e', 'ssh -x', x,
-                        '%s,%s@frs.sourceforge.net:%s' %
-                        (self.username, self.project, self.rdir + '/')
+                        '{},{}@frs.sourceforge.net:{}'.format(self.username, self.project, self.rdir + '/')
                     ])
                 except KeyboardInterrupt:
                     raise SystemExit(1)
@@ -123,7 +125,6 @@ class SourceForge(Base):  # {{{
                     break
             print('Uploaded in', int(time.time() - start), 'seconds\n\n')
 
-
 # }}}
 
 
@@ -149,21 +150,16 @@ class GitHub(Base):  # {{{
         existing_assets = self.existing_assets(release['id'])
         for path, desc in self.files.items():
             self.info('')
-            url = self.API + 'repos/{}/{}/releases/assets/{{}}'.format(
-                self.username, self.reponame
-            )
+            url = self.API + f'repos/{self.username}/{self.reponame}/releases/assets/{{}}'
             fname = os.path.basename(path)
             if fname in existing_assets:
-                self.info(
-                    'Deleting %s from GitHub with id: %s' %
-                    (fname, existing_assets[fname])
-                )
+                self.info(f'Deleting {fname} from GitHub with id: {existing_assets[fname]}')
                 r = self.requests.delete(url.format(existing_assets[fname]))
                 if r.status_code != 204:
-                    self.fail(r, 'Failed to delete %s from GitHub' % fname)
+                    self.fail(r, f'Failed to delete {fname} from GitHub')
             r = self.do_upload(upload_url, path, desc, fname)
             if r.status_code != 201:
-                self.fail(r, 'Failed to upload file: %s' % fname)
+                self.fail(r, f'Failed to upload file: {fname}')
             try:
                 r = self.requests.patch(
                     url.format(r.json()['id']),
@@ -182,25 +178,22 @@ class GitHub(Base):  # {{{
                     })
                 )
             if r.status_code != 200:
-                self.fail(r, 'Failed to set label for %s' % fname)
+                self.fail(r, f'Failed to set label for {fname}')
 
     def clean_older_releases(self, releases):
         for release in releases:
             if release.get('assets',
                            None) and release['tag_name'] != self.current_tag_name:
                 self.info(
-                    '\nDeleting old released installers from: %s' %
-                    release['tag_name']
+                    '\nDeleting old released installers from: {}'.format(release['tag_name'])
                 )
                 for asset in release['assets']:
                     r = self.requests.delete(
-                        self.API + 'repos/%s/%s/releases/assets/%s' %
-                        (self.username, self.reponame, asset['id'])
+                        self.API + 'repos/{}/{}/releases/assets/{}'.format(self.username, self.reponame, asset['id'])
                     )
                     if r.status_code != 204:
                         self.fail(
-                            r, 'Failed to delete obsolete asset: %s for release: %s'
-                            % (asset['name'], release['tag_name'])
+                            r, 'Failed to delete obsolete asset: {} for release: {}'.format(asset['name'], release['tag_name'])
                         )
 
     def do_upload(self, url, path, desc, fname):
@@ -218,8 +211,8 @@ class GitHub(Base):  # {{{
             )
 
     def fail(self, r, msg):
-        print(msg, ' Status Code: %s' % r.status_code, file=sys.stderr)
-        print("JSON from response:", file=sys.stderr)
+        print(msg, f' Status Code: {r.status_code}', file=sys.stderr)
+        print('JSON from response:', file=sys.stderr)
         pprint(dict(r.json()), stream=sys.stderr)
         raise SystemExit(1)
 
@@ -228,9 +221,7 @@ class GitHub(Base):  # {{{
         return error_code == 'already_exists'
 
     def existing_assets(self, release_id):
-        url = self.API + 'repos/{}/{}/releases/{}/assets'.format(
-            self.username, self.reponame, release_id
-        )
+        url = self.API + f'repos/{self.username}/{self.reponame}/releases/{release_id}/assets'
         r = self.requests.get(url)
         if r.status_code != 200:
             self.fail('Failed to get assets for release')
@@ -255,17 +246,16 @@ class GitHub(Base):  # {{{
             data=json.dumps({
                 'tag_name': self.current_tag_name,
                 'target_commitish': 'master',
-                'name': 'version %s' % self.version,
-                'body': 'Release version %s' % self.version,
+                'name': f'version {self.version}',
+                'body': f'Release version {self.version}',
                 'draft': False,
                 'prerelease': False
             })
         )
         if r.status_code != 201:
-            self.fail(r, 'Failed to create release for version: %s' % self.version)
+            self.fail(r, f'Failed to create release for version: {self.version}')
         return r.json()
 
-
 # }}}
 
 
@@ -282,7 +272,7 @@ def generate_index():  # {{{
             rmap[series] = []
         rmap[series].append(rnum)
 
-    template = '''<!DOCTYPE html>\n<html lang="en"> <head> <meta charset="utf-8"> <title>{title}</title><link rel="icon" type="image/png" href="//calibre-ebook.com/favicon.png" /> <style type="text/css"> {style} </style> </head> <body> <h1>{title}</h1> <p>{msg}</p> {body} </body> </html> '''  # noqa
+    template = '''<!DOCTYPE html>\n<html lang="en"> <head> <meta charset="utf-8"> <title>{title}</title><link rel="icon" type="image/png" href="//calibre-ebook.com/favicon.png" /> <style type="text/css"> {style} </style> </head> <body> <h1>{title}</h1> <p>{msg}</p> {body} </body> </html> '''  # noqa: E501
     style = '''
     body { font-family: sans-serif; background-color: #eee; }
     a { text-decoration: none; }
@@ -298,10 +288,8 @@ def generate_index():  # {{{
     body = []
     for series in rmap:
         body.append(
-            '<li><a href="{0}.html" title="Releases in the {0}.x series">{0}.x</a>\xa0\xa0\xa0<span style="font-size:smaller">[{1} releases]</span></li>'
-            .format(  # noqa
-                '.'.join(map(str, series)), len(rmap[series])
-            )
+            '<li><a href="{0}.html" title="Releases in the {0}.x series">{0}.x</a>\xa0\xa0\xa0<span style="font-size:smaller">[{1} releases]</span></li>'  # noqa: E501
+            .format('.'.join(map(str, series)), len(rmap[series]))
         )
     body = '<ul>{}</ul>'.format(' '.join(body))
     index = template.format(
@@ -322,12 +310,12 @@ def generate_index():  # {{{
         ]
         body = '<ul class="release-list">{}</ul>'.format(' '.join(body))
         index = template.format(
-            title='Previous calibre releases (%s.x)' % sname,
+            title=f'Previous calibre releases ({sname}.x)',
             style=style,
             msg='Choose a calibre release',
             body=body
         )
-        with open('%s.html' % sname, 'wb') as f:
+        with open(f'{sname}.html', 'wb') as f:
             f.write(index.encode('utf-8'))
 
         for r in releases:
@@ -338,10 +326,10 @@ def generate_index():  # {{{
                 files = os.listdir('.')
                 windows = [x for x in files if x.endswith('.msi')]
                 if windows:
+                    def wdesc(x):
+                        return 'Windows ' + ('64-bit' if '-64bit-' in x else '32-bit') + ' Installer'
                     windows = [
-                        '<li><a href="{0}" title="{1}">{1}</a></li>'.format(
-                            x, 'Windows 64-bit Installer'
-                        ) for x in windows
+                        '<li><a href="{0}" title="{1}">{1}</a></li>'.format(x, wdesc(x)) for x in windows
                     ]
                     body.append(
                         '<dt>Windows</dt><dd><ul>{}</ul></dd>'.format(
@@ -361,21 +349,24 @@ def generate_index():  # {{{
                         .format(osx[0], 'OS X Disk Image (.dmg)')
                     )
                 linux = [
-                    x for x in files if x.endswith('.txz') or x.endswith('tar.bz2')
+                    x for x in files if x.endswith(('.txz', 'tar.bz2'))
                 ]
                 if linux:
+                    def ldesc(x):
+                        if 'i686' in x:
+                            return 'Linux Intel 32-bit binary'
+                        if 'arm64' in x:
+                            return 'Linux ARM 64-bit binary'
+                        return 'Linux Intel 64-bit binary'
                     linux = [
-                        '<li><a href="{0}" title="{1}">{1}</a></li>'.format(
-                            x, 'Linux 64-bit binary'
-                            if 'x86_64' in x else 'Linux ARM 64-bit binary'
-                        ) for x in linux
+                        '<li><a href="{0}" title="{1}">{1}</a></li>'.format(x, ldesc(x)) for x in linux
                     ]
                     body.append(
                         '<dt>Linux</dt><dd><ul>{}</ul></dd>'.format(
                             ' '.join(linux)
                         )
                     )
-                source = [x for x in files if x.endswith('.xz') or x.endswith('.gz')]
+                source = [x for x in files if x.endswith(('.xz', '.gz'))]
                 if source:
                     body.append(
                         '<dt>Source Code</dt><dd><a href="{0}" title="{1}">{1}</a></dd>'
@@ -384,7 +375,7 @@ def generate_index():  # {{{
 
                 body = '<dl>{}</dl>'.format(''.join(body))
                 index = template.format(
-                    title='calibre release (%s)' % rname,
+                    title=f'calibre release ({rname})',
                     style=style,
                     msg='',
                     body=body
@@ -393,10 +384,9 @@ def generate_index():  # {{{
                     f.write(index.encode('utf-8'))
             finally:
                 os.chdir('..')
-
-
 # }}}
 
+
 SERVER_BASE = '/srv/download/'
 
 
@@ -415,31 +405,28 @@ def upload_to_servers(files, version):  # {{{
 
     # for server, rdir in {'files':'/srv/download/'}.items():
     #     print('Uploading to server:', server)
-    #     server = '%s.calibre-ebook.com' % server
+    #     server = f'{server}.calibre-ebook.com'
     #     # Copy the generated index files
-    #     print ('Copying generated index')
+    #     print('Copying generated index')
     #     check_call(['rsync', '-hza', '-e', 'ssh -x', '--include', '*.html',
-    #                 '--filter', '-! */', base, 'root@%s:%s' % (server, rdir)])
+    #                 '--filter', '-! */', base, f'root@{server}:{rdir}'])
     #     # Copy the release files
-    #     rdir = '%s%s/' % (rdir, version)
+    #     rdir = f'{rdir}{version}/'
     #     for x in files:
     #         start = time.time()
-    #         print ('Uploading', x)
+    #         print('Uploading', x)
     #         for i in range(5):
     #             try:
     #                 check_call(['rsync', '-h', '-z', '--progress', '-e', 'ssh -x', x,
-    #                 'root@%s:%s'%(server, rdir)])
+    #                 f'root@{server}:{rdir}'])
     #             except KeyboardInterrupt:
     #                 raise SystemExit(1)
     #             except:
-    #                 print ('\nUpload failed, trying again in 30 seconds')
+    #                 print('\nUpload failed, trying again in 30 seconds')
     #                 time.sleep(30)
     #             else:
     #                 break
-    #         print ('Uploaded in', int(time.time() - start), 'seconds\n\n')
-    #
-
-
+    #         print('Uploaded in', int(time.time() - start), 'seconds\n\n')
 # }}}
 
 
