@@ -5,11 +5,17 @@ __license__ = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, shutil, subprocess, tempfile, json, time, filecmp, sys
+import filecmp
+import json
+import os
+import shutil
+import subprocess
+import sys
+import tempfile
+import time
 
-from setup import Command, __version__, require_clean_git, require_git_master
-from setup.upload import installers
-from setup.parallel_build import parallel_build, create_job
+from setup import Command, __version__, installer_names, require_clean_git, require_git_master
+from setup.parallel_build import create_job, parallel_build
 
 
 class Stage1(Command):
@@ -41,12 +47,12 @@ class Stage2(Command):
         self.info('Starting builds for all platforms, this will take a while...')
 
         session = ['layout vertical']
-        platforms = 'linux', 'osx', 'win'
+        platforms = 'linux64', 'linuxarm64', 'osx', 'win'
         for x in platforms:
             cmd = (
-                '''{exe} -c "import subprocess; subprocess.Popen(['{exe}', './setup.py', '{x}']).wait() != 0 and'''
-                ''' input('Build of {x} failed, press Enter to exit');"'''
-            ).format(exe=sys.executable, x=x)
+                f'''{sys.executable} -c "import subprocess; subprocess.Popen(['{sys.executable}', './setup.py', '{x}']).wait() != 0 and'''
+                f''' input('Build of {x} failed, press Enter to exit');"'''
+            )
             session.append('title ' + x)
             session.append('launch ' + cmd)
 
@@ -58,12 +64,10 @@ class Stage2(Command):
         p.communicate('\n'.join(session).encode('utf-8'))
         p.wait()
 
-        for installer in installers(include_source=False):
+        for installer in installer_names(include_source=False):
             installer = self.j(self.d(self.SRC), installer)
             if not os.path.exists(installer) or os.path.getsize(installer) < 10000:
-                raise SystemExit(
-                    'The installer %s does not exist' % os.path.basename(installer)
-                )
+                raise SystemExit(f'The installer {os.path.basename(installer)} does not exist')
 
 
 class Stage3(Command):
@@ -101,10 +105,14 @@ class Publish(Command):
     def pre_sub_commands(self, opts):
         require_git_master()
         require_clean_git()
+        version = tuple(map(int, __version__.split('.')))  # noqa: RUF048
+        if version[2] > 99:
+            raise SystemExit(f'The version number {__version__} indicates a preview release, did you mean to run ./setup.py publish_preview?')
         if 'PUBLISH_BUILD_DONE' not in os.environ:
             subprocess.check_call([sys.executable, 'setup.py', 'check'])
             subprocess.check_call([sys.executable, 'setup.py', 'build'])
-            subprocess.check_call([sys.executable, 'setup.py', 'test'])
+            if 'SKIP_CALIBRE_TESTS' not in os.environ:
+                subprocess.check_call([sys.executable, 'setup.py', 'test'])
             subprocess.check_call([sys.executable, 'setup.py', 'pot'])
             subprocess.check_call([sys.executable, 'setup.py', 'translations'])
             os.environ['PUBLISH_BUILD_DONE'] = '1'
@@ -122,8 +130,31 @@ class PublishBetas(Command):
     def run(self, opts):
         dist = self.a(self.j(self.d(self.SRC), 'dist'))
         subprocess.check_call((
-            'rsync --partial -rh --info=progress2 --delete-after %s/ download.calibre-ebook.com:/srv/download/betas/'
-            % dist
+            f'rsync --partial -rh --info=progress2 --delete-after {dist}/ download.calibre-ebook.com:/srv/download/betas/'
+        ).split())
+
+
+class PublishPreview(Command):
+
+    sub_commands = ['stage1', 'stage2', 'sdist']
+
+    def pre_sub_commands(self, opts):
+        version = tuple(map(int, __version__.split('.')))  # noqa: RUF048
+        if version[2] < 100:
+            raise SystemExit('Must set calibre version to have patch level greater than 100')
+        require_clean_git()
+        require_git_master()
+
+    def run(self, opts):
+        dist = self.a(self.j(self.d(self.SRC), 'dist'))
+        with open(os.path.join(dist, 'README.txt'), 'w') as f:
+            print('''\
+These are preview releases of changes to calibre since the last normal release.
+Preview releases are typically released every Friday, they serve as a way
+for users to test upcoming features/fixes in the next calibre release.
+''', file=f)
+        subprocess.check_call((
+            f'rsync -rh --info=progress2 --delete-after --delete {dist}/ download.calibre-ebook.com:/srv/download/preview/'
         ).split())
 
 
@@ -164,14 +195,16 @@ class Manual(Command):
         languages = opts.language or list(
             json.load(open(self.j(base, 'locale', 'completed.json'), 'rb'))
         )
-        languages = ['en'] + list(set(languages) - {'en', 'ja'})
+        languages = set(languages) - {'en'}
+        languages.discard('ta')  # Tamil translations break Sphinx
+        languages = ['en'] + list(languages)
         os.environ['ALL_USER_MANUAL_LANGUAGES'] = ' '.join(languages)
         for language in languages:
             jobs.append(create_job([
                 sys.executable, self.j(self.d(self.SRC), 'manual', 'build.py'),
                 language, self.j(tdir, language)
-            ], '\n\n**************** Building translations for: %s' % language))
-        self.info('Building manual for %d languages' % len(jobs))
+            ], f'\n\n**************** Building translations for: {language}'))
+        self.info(f'Building manual for {len(jobs)} languages')
         subprocess.check_call(jobs[0].cmd)
         if not parallel_build(jobs[1:], self.info):
             raise SystemExit(1)
@@ -190,8 +223,8 @@ class Manual(Command):
                 if x and not os.path.exists(x):
                     os.symlink('.', x)
             self.info(
-                'Built manual for %d languages in %s minutes' %
-                (len(jobs), int((time.time() - st) / 60.))
+                f'Built manual for {len(jobs)} languages in {int((time.time() - st) / 60.)} minutes'
+
             )
         finally:
             os.chdir(cwd)
@@ -204,13 +237,13 @@ class Manual(Command):
         from polyglot.http_server import HTTPServer, SimpleHTTPRequestHandler
         HandlerClass = SimpleHTTPRequestHandler
         ServerClass = HTTPServer
-        Protocol = "HTTP/1.0"
+        Protocol = 'HTTP/1.0'
         server_address = ('127.0.0.1', 8000)
 
         HandlerClass.protocol_version = Protocol
         httpd = ServerClass(server_address, HandlerClass)
 
-        print("Serving User Manual on localhost:8000")
+        print('Serving User Manual on localhost:8000')
         from calibre.gui2 import open_url
         open_url('http://localhost:8000')
         httpd.serve_forever()
@@ -254,8 +287,10 @@ class ManPages(Command):
             shutil.rmtree(dest)
         os.makedirs(dest)
         base = self.j(self.d(self.SRC), 'manual')
-        languages = list(available_translations())
-        languages = ['en'] + list(set(languages) - {'en', 'en_GB'})
+        languages = set(available_translations())
+        languages.discard('ta')  # Tamil translatins are completely borked break sphinx
+        languages.discard('id')  # Indonesian man page fails to build
+        languages = ['en'] + list(languages - {'en', 'en_GB'})
         os.environ['ALL_USER_MANUAL_LANGUAGES'] = ' '.join(languages)
         try:
             os.makedirs(dest)
@@ -265,7 +300,7 @@ class ManPages(Command):
         for l in languages:
             jobs.append(create_job(
                 [sys.executable, self.j(base, 'build.py'), '--man-pages', l, dest],
-                '\n\n**************** Building translations for: %s' % l)
+                f'\n\n**************** Building translations for: {l}')
             )
         self.info(f'\tCreating man pages in {dest} for {len(jobs)} languages...')
         subprocess.check_call(jobs[0].cmd)
@@ -304,6 +339,6 @@ class TagRelease(Command):
     def run(self, opts):
         self.info('Tagging release')
         subprocess.check_call(
-            'git tag -s v{0} -m "version-{0}"'.format(__version__).split()
+            f'git tag -s v{__version__} -m "version-{__version__}"'.split()
         )
         subprocess.check_call(f'git push origin v{__version__}'.split())
