@@ -1,18 +1,19 @@
 __license__   = 'GPL v3'
-__copyright__ = '2010-2012, , Timothy Legge <timlegge at gmail.com> and David Forrester <davidfor@internode.on.net>'
+__copyright__ = '2010-2012, Timothy Legge <timlegge at gmail.com> and David Forrester <davidfor@internode.on.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, time, sys
-from functools import cmp_to_key
+import os
+import time
 
-from calibre.constants import preferred_encoding, DEBUG
 from calibre import isbytestring
-
-from calibre.ebooks.metadata.book.base import Metadata
-from calibre.devices.usbms.books import Book as Book_, CollectionsBookList, none_cmp
-from calibre.utils.config_base import prefs
-from calibre.devices.usbms.driver import debug_print
+from calibre.constants import DEBUG, preferred_encoding
+from calibre.devices.usbms.books import Book as Book_
+from calibre.devices.usbms.books import CollectionsBookList
 from calibre.ebooks.metadata import author_to_author_sort
+from calibre.ebooks.metadata.book.base import Metadata
+from calibre.ebooks.metadata.book.formatter import SafeFormat
+from calibre.prints import debug_print
+from calibre.utils.config_base import prefs
 
 
 class Book(Book_):
@@ -20,14 +21,14 @@ class Book(Book_):
     def __init__(self, prefix, lpath, title=None, authors=None, mime=None, date=None, ContentType=None,
                  thumbnail_name=None, size=None, other=None):
         from calibre.utils.date import parse_date
-#         debug_print('Book::__init__ - title=', title)
-        show_debug = title is not None and title.lower().find("xxxxx") >= 0
+        # debug_print('Book::__init__ - title=', title)
+        show_debug = title is not None and title.lower().find('xxxxx') >= 0
         if other is not None:
             other.title = title
             other.published_date = date
         if show_debug:
-            debug_print("Book::__init__ - title=", title, 'authors=', authors)
-            debug_print("Book::__init__ - other=", other)
+            debug_print('Book::__init__ - title=', title, 'authors=', authors)
+            debug_print('Book::__init__ - other=', other)
         super().__init__(prefix, lpath, size, other)
 
         if title is not None and len(title) > 0:
@@ -35,7 +36,7 @@ class Book(Book_):
 
         if authors is not None and len(authors) > 0:
             self.authors_from_string(authors)
-            if self.author_sort is None or self.author_sort == "Unknown":
+            if self.author_sort is None or self.author_sort == 'Unknown':
                 self.author_sort = author_to_author_sort(authors)
 
         self.mime = mime
@@ -44,13 +45,13 @@ class Book(Book_):
 
         if ContentType == '6' and date is not None:
             try:
-                self.datetime = time.strptime(date, "%Y-%m-%dT%H:%M:%S.%f")
+                self.datetime = time.strptime(date, '%Y-%m-%dT%H:%M:%S.%f')
             except:
                 try:
-                    self.datetime = time.strptime(date.split('+')[0], "%Y-%m-%dT%H:%M:%S")
+                    self.datetime = time.strptime(date.split('+')[0], '%Y-%m-%dT%H:%M:%S')
                 except:
                     try:
-                        self.datetime = time.strptime(date.split('+')[0], "%Y-%m-%d")
+                        self.datetime = time.strptime(date.split('+')[0], '%Y-%m-%d')
                     except:
                         try:
                             self.datetime = parse_date(date,
@@ -76,13 +77,13 @@ class Book(Book_):
             self.thumbnail = ImageWrapper(thumbnail_name)
 
         if show_debug:
-            debug_print("Book::__init__ end - self=", self)
-            debug_print("Book::__init__ end - title=", title, 'authors=', authors)
+            debug_print('Book::__init__ end - self=', self)
+            debug_print('Book::__init__ end - title=', title, 'authors=', authors)
 
     @property
     def is_sideloaded(self):
         # If we don't have a content Id, we don't know what type it is.
-        return self.contentID and self.contentID.startswith("file")
+        return self.contentID and self.contentID.startswith('file')
 
     @property
     def has_kobo_series(self):
@@ -90,22 +91,22 @@ class Book(Book_):
 
     @property
     def is_purchased_kepub(self):
-        return self.contentID and not self.contentID.startswith("file")
+        return self.contentID and not self.contentID.startswith('file')
 
     def __str__(self):
         '''
         A string representation of this object, suitable for printing to
         console
         '''
-        ans = ["Kobo metadata:"]
+        ans = ['Kobo metadata:']
 
         def fmt(x, y):
-            ans.append('%-20s: %s'%(str(x), str(y)))
+            ans.append(f'{x:<20}: {y}')
 
         if self.contentID:
             fmt('Content ID', self.contentID)
         if self.kobo_series:
-            fmt('Kobo Series', self.kobo_series + ' #%s'%self.kobo_series_number)
+            fmt('Kobo Series', self.kobo_series + f' #{self.kobo_series_number}')
         if self.kobo_series_id:
             fmt('Kobo Series ID', self.kobo_series_id)
         if self.kobo_subtitle:
@@ -117,7 +118,7 @@ class Book(Book_):
 
         ans = '\n'.join(ans)
 
-        return super().__str__() + "\n" + ans
+        return super().__str__() + '\n' + ans
 
 
 class ImageWrapper:
@@ -132,8 +133,8 @@ class KTCollectionsBookList(CollectionsBookList):
         super().__init__(oncard, prefix, settings)
         self.set_device_managed_collections([])
 
-    def get_collections(self, collection_attributes):
-        debug_print("KTCollectionsBookList:get_collections - start - collection_attributes=", collection_attributes)
+    def get_collections(self, collection_attributes, collections_template=None, template_globals=None):
+        debug_print('KTCollectionsBookList:get_collections - start - collection_attributes=', collection_attributes)
 
         collections = {}
 
@@ -141,7 +142,7 @@ class KTCollectionsBookList(CollectionsBookList):
         for c in collection_attributes:
             ca.append(c.lower())
         collection_attributes = ca
-        debug_print("KTCollectionsBookList:get_collections - collection_attributes=", collection_attributes)
+        debug_print('KTCollectionsBookList:get_collections - collection_attributes=', collection_attributes)
 
         for book in self:
             tsval = book.get('title_sort', book.title)
@@ -150,21 +151,24 @@ class KTCollectionsBookList(CollectionsBookList):
 
             show_debug = self.is_debugging_title(tsval) or tsval is None
             if show_debug:  # or len(book.device_collections) > 0:
-                debug_print('KTCollectionsBookList:get_collections - tsval=', tsval, "book.title=", book.title, "book.title_sort=", book.title_sort)
+                debug_print('KTCollectionsBookList:get_collections - tsval=', tsval, 'book.title=', book.title, 'book.title_sort=', book.title_sort)
                 debug_print('KTCollectionsBookList:get_collections - book.device_collections=', book.device_collections)
                 # debug_print(book)
             # Make sure we can identify this book via the lpath
             lpath = getattr(book, 'lpath', None)
             if lpath is None:
                 continue
-            # If the book is not in the current library, we don't want to use the metadtaa for the collections
-            if book.application_id is None:
-                # debug_print("KTCollectionsBookList:get_collections - Book not in current library")
+            # If the book is not in the current library, we don't want to use the metadata for the collections
+            # or it is a book that cannot be put in a collection (such as recommendations or previews)
+            if book.application_id is None or not book.can_put_on_shelves:
+                # debug_print("KTCollectionsBookList:get_collections - Book not in current library or cannot be put in a collection")
                 continue
+
             # Decide how we will build the collections. The default: leave the
             # book in all existing collections. Do not add any new ones.
             attrs = ['device_collections']
             if getattr(book, '_new_book', False):
+                debug_print('KTCollectionsBookList:get_collections - sending new book')
                 if prefs['manage_device_metadata'] == 'manual':
                     # Ensure that the book is in all the book's existing
                     # collections plus all metadata collections
@@ -173,29 +177,34 @@ class KTCollectionsBookList(CollectionsBookList):
                     # For new books, both 'on_send' and 'on_connect' do the same
                     # thing. The book's existing collections are ignored. Put
                     # the book in collections defined by its metadata.
-                    attrs = collection_attributes
+                    attrs = list(collection_attributes)
             elif prefs['manage_device_metadata'] == 'on_connect':
                 # For existing books, modify the collections only if the user
                 # specified 'on_connect'
-                attrs = collection_attributes
+                attrs = list(collection_attributes)
                 for cat_name in self.device_managed_collections:
                     if cat_name in book.device_collections:
                         if cat_name not in collections:
                             collections[cat_name] = {}
                             if show_debug:
-                                debug_print("KTCollectionsBookList:get_collections - Device Managed Collection:", cat_name)
+                                debug_print('KTCollectionsBookList:get_collections - Device Managed Collection:', cat_name)
                         if lpath not in collections[cat_name]:
-                            collections[cat_name][lpath] = (book, tsval, tsval)
+                            collections[cat_name][lpath] = book
                             if show_debug:
-                                debug_print("KTCollectionsBookList:get_collections - Device Managed Collection -added book to cat_name", cat_name)
+                                debug_print('KTCollectionsBookList:get_collections - Device Managed Collection -added book to cat_name', cat_name)
                 book.device_collections = []
             if show_debug:
-                debug_print("KTCollectionsBookList:get_collections - attrs=", attrs)
+                debug_print('KTCollectionsBookList:get_collections - attrs=', attrs)
+
+            if collections_template is not None:
+                attrs.append('%template%')
 
             for attr in attrs:
+                fm = None
                 attr = attr.strip()
                 if show_debug:
-                    debug_print("KTCollectionsBookList:get_collections - attr='%s'"%attr)
+                    debug_print(f"KTCollectionsBookList:get_collections - attr='{attr}'")
+
                 # If attr is device_collections, then we cannot use
                 # format_field, because we don't know the fields where the
                 # values came from.
@@ -203,19 +212,26 @@ class KTCollectionsBookList(CollectionsBookList):
                     doing_dc = True
                     val = book.device_collections  # is a list
                     if show_debug:
-                        debug_print("KTCollectionsBookList:get_collections - adding book.device_collections", book.device_collections)
-                # If the book is not in the current library, we don't want to use the metadtaa for the collections
-                elif book.application_id is None or not book.can_put_on_shelves:
-                    # debug_print("KTCollectionsBookList:get_collections - Book not in current library")
-                    continue
+                        debug_print('KTCollectionsBookList:get_collections - adding book.device_collections', book.device_collections)
+                elif attr == '%template%':
+                    doing_dc = False
+                    val = ''
+                    if collections_template is not None:
+                        nv = SafeFormat().safe_format(collections_template, book,
+                                                      'KOBO', book, global_vars=template_globals)
+                        if show_debug:
+                            debug_print('KTCollectionsBookList:get_collections collections_template - result', nv)
+                        if nv:
+                            val = [v.strip() for v in nv.split(':@:') if v.strip()]
                 else:
                     doing_dc = False
                     ign, val, orig_val, fm = book.format_field_extended(attr)
                     val = book.get(attr, None)
                     if show_debug:
-                        debug_print("KTCollectionsBookList:get_collections - not device_collections")
+                        debug_print('KTCollectionsBookList:get_collections - not device_collections')
                         debug_print('          ign=', ign, ', val=', val, ' orig_val=', orig_val, 'fm=', fm)
                         debug_print('          val=', val)
+
                 if not val:
                     continue
                 if isbytestring(val):
@@ -233,39 +249,29 @@ class KTCollectionsBookList(CollectionsBookList):
                     else:
                         val = [orig_val]
                     if show_debug:
-                        debug_print("KTCollectionsBookList:get_collections - val is text and multiple", val)
+                        debug_print('KTCollectionsBookList:get_collections - val is text and multiple', val)
                 elif fm is not None and fm['datatype'] == 'composite' and fm['is_multiple']:
                     if show_debug:
-                        debug_print("KTCollectionsBookList:get_collections - val is compositeand multiple", val)
+                        debug_print('KTCollectionsBookList:get_collections - val is compositeand multiple', val)
                     val = [v.strip() for v in
                            val.split(fm['is_multiple']['ui_to_list'])]
                 else:
                     val = [val]
                 if show_debug:
-                    debug_print("KTCollectionsBookList:get_collections - val=", val)
+                    debug_print('KTCollectionsBookList:get_collections - val=', val)
 
                 for category in val:
                     # debug_print("KTCollectionsBookList:get_collections - category=", category)
-                    is_series = False
                     if doing_dc:
-                        # Attempt to determine if this value is a series by
-                        # comparing it to the series name.
-                        if category == book.series:
-                            is_series = True
+                        pass  # No need to do anything with device_collections
                     elif fm is not None and fm['is_custom']:  # is a custom field
                         if fm['datatype'] == 'text' and len(category) > 1 and \
                                 category[0] == '[' and category[-1] == ']':
                             continue
-                        if fm['datatype'] == 'series':
-                            is_series = True
                     else:                       # is a standard field
                         if attr == 'tags' and len(category) > 1 and \
                                 category[0] == '[' and category[-1] == ']':
                             continue
-                        if attr == 'series' or \
-                                ('series' in collection_attributes and
-                                 book.get('series', None) == category):
-                            is_series = True
 
                     # The category should not be None, but, it has happened.
                     if not category:
@@ -276,30 +282,21 @@ class KTCollectionsBookList(CollectionsBookList):
                     if cat_name not in collections:
                         collections[cat_name] = {}
                         if show_debug:
-                            debug_print("KTCollectionsBookList:get_collections - created collection for cat_name", cat_name)
+                            debug_print('KTCollectionsBookList:get_collections - created collection for cat_name', cat_name)
                     if lpath not in collections[cat_name]:
-                        if is_series:
-                            if doing_dc:
-                                collections[cat_name][lpath] = \
-                                    (book, book.get('series_index', sys.maxsize), tsval)
-                            else:
-                                collections[cat_name][lpath] = \
-                                    (book, book.get(attr+'_index', sys.maxsize), tsval)
-                        else:
-                            collections[cat_name][lpath] = (book, tsval, tsval)
+                        collections[cat_name][lpath] = book
                         if show_debug:
-                            debug_print("KTCollectionsBookList:get_collections - added book to collection for cat_name", cat_name)
+                            debug_print('KTCollectionsBookList:get_collections - added book to collection for cat_name', cat_name)
                     if show_debug:
-                        debug_print("KTCollectionsBookList:get_collections - cat_name", cat_name)
+                        debug_print('KTCollectionsBookList:get_collections - cat_name', cat_name)
 
         # Sort collections
         result = {}
 
         for category, lpaths in collections.items():
-            books = sorted(lpaths.values(), key=cmp_to_key(none_cmp))
-            result[category] = [x[0] for x in books]
+            result[category] = lpaths.values()
         # debug_print("KTCollectionsBookList:get_collections - result=", result.keys())
-        debug_print("KTCollectionsBookList:get_collections - end")
+        debug_print('KTCollectionsBookList:get_collections - end')
         return result
 
     def set_device_managed_collections(self, collection_names):
@@ -311,9 +308,9 @@ class KTCollectionsBookList(CollectionsBookList):
     def is_debugging_title(self, title):
         if not DEBUG:
             return False
-#        debug_print("KTCollectionsBookList:is_debugging - title=", title, "self.debugging_title=", self.debugging_title)
+        # debug_print("KTCollectionsBookList:is_debugging - title=", title, "self.debugging_title=", self.debugging_title)
         is_debugging = self.debugging_title is not None and len(self.debugging_title) > 0 and title is not None and (
             title.lower().find(self.debugging_title.lower()) >= 0 or len(title) == 0)
-#        debug_print("KTCollectionsBookList:is_debugging - is_debugging=", is_debugging)
+        # debug_print("KTCollectionsBookList:is_debugging - is_debugging=", is_debugging)
 
         return is_debugging
