@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 
 __license__   = 'GPL v3'
@@ -9,16 +8,15 @@ __docformat__ = 'restructuredtext en'
 import struct
 from collections import OrderedDict, namedtuple
 
-from calibre.ebooks.mobi.utils import (decint, count_set_bits,
-        decode_string)
-from polyglot.builtins import iteritems, range, zip
+from calibre.ebooks.mobi.utils import count_set_bits, decint, decode_string
+from polyglot.builtins import iteritems
 
 TagX = namedtuple('TagX', 'tag num_of_values bitmask eof')
 PTagX = namedtuple('PTagX', 'tag value_count value_bytes num_of_values')
 INDEX_HEADER_FIELDS = (
             'len', 'nul1', 'type', 'gen', 'start', 'count', 'code',
             'lng', 'total', 'ordt', 'ligt', 'nligt', 'ncncx'
-    ) + tuple('unknown%d'%i for i in range(27)) + ('ocnt', 'oentries',
+    ) + tuple(f'unknown{i}' for i in range(27)) + ('ocnt', 'oentries',
             'ordt1', 'ordt2', 'tagx')
 
 
@@ -28,7 +26,7 @@ class InvalidFile(ValueError):
 
 def check_signature(data, signature):
     if data[:len(signature)] != signature:
-        raise InvalidFile('Not a valid %r section'%signature)
+        raise InvalidFile(f'Not a valid {signature!r} section')
 
 
 class NotAnINDXRecord(InvalidFile):
@@ -49,14 +47,15 @@ def parse_indx_header(data):
     check_signature(data, b'INDX')
     words = INDEX_HEADER_FIELDS
     num = len(words)
-    values = struct.unpack('>%dL' % num, data[4:4*(num+1)])
+    values = struct.unpack(f'>{num}L', data[4:4*(num+1)])
     ans = dict(zip(words, values))
+    ans['idx_header_end_pos'] = 4 * (num+1)
     ordt1, ordt2 = ans['ordt1'], ans['ordt2']
     ans['ordt1_raw'], ans['ordt2_raw'] = [], []
     ans['ordt_map'] = ''
 
     if ordt1 > 0 and data[ordt1:ordt1+4] == b'ORDT':
-        # I dont know what this is, but using it seems to be unnecessary, so
+        # I don't know what this is, but using it seems to be unnecessary, so
         # just leave it as the raw bytestring
         ans['ordt1_raw'] = data[ordt1+4:ordt1+4+ans['oentries']]
     if ordt2 > 0 and data[ordt2:ordt2+4] == b'ORDT':
@@ -104,8 +103,7 @@ class CNCX:  # {{{
                     except:
                         byts = raw[pos:]
                         r = format_bytes(byts)
-                        print('CNCX entry at offset %d has unknown format %s'%(
-                            pos+record_offset, r))
+                        print(f'CNCX entry at offset {pos + record_offset} has unknown format {r}')
                         self.records[pos+record_offset] = r
                         pos = len(raw)
                 pos += consumed+length
@@ -194,8 +192,7 @@ def get_tag_map(control_byte_count, tagx, data, strict=False):
                 total_consumed += consumed
                 values.append(byts)
             if total_consumed != x.value_bytes:
-                err = ("Error: Should consume %s bytes, but consumed %s" %
-                        (x.value_bytes, total_consumed))
+                err = (f'Error: Should consume {x.value_bytes} bytes, but consumed {total_consumed}')
                 if strict:
                     raise ValueError(err)
                 else:
@@ -203,8 +200,7 @@ def get_tag_map(control_byte_count, tagx, data, strict=False):
         ans[x.tag] = values
     # Test that all bytes have been processed
     if data.replace(b'\0', b''):
-        err = ("Warning: There are unprocessed index bytes left: %s" %
-                format_bytes(data))
+        err = (f'Warning: There are unprocessed index bytes left: {format_bytes(data)}')
         if strict:
             raise ValueError(err)
         else:
@@ -241,18 +237,27 @@ def parse_index_record(table, data, control_byte_count, tags, codec,
             ident, consumed = decode_string(rec, codec=codec, ordt_map=ordt_map)
         except UnicodeDecodeError:
             ident, consumed = decode_string(rec, codec='utf-16', ordt_map=ordt_map)
-        if u'\x00' in ident:
+        if '\x00' in ident:
             try:
                 ident, consumed = decode_string(rec, codec='utf-16',
                         ordt_map=ordt_map)
             except UnicodeDecodeError:
-                ident = ident.replace('u\x00', u'')
+                ident = ident.replace('u\x00', '')
         rec = rec[consumed:]
         tag_map = get_tag_map(control_byte_count, tags, rec, strict=strict)
         table[ident] = tag_map
     return header
 
 
+def get_tag_section_start(data, indx_header):
+    tag_section_start = indx_header['tagx']
+    if data[tag_section_start:tag_section_start + 4] != b'TAGX':
+        tpos = data.find(b'TAGX', indx_header['idx_header_end_pos'])
+        if tpos > -1:
+            tag_section_start = tpos
+    return tag_section_start
+
+
 def read_index(sections, idx, codec):
     table, cncx = OrderedDict(), CNCX([], codec)
 
@@ -266,7 +271,7 @@ def read_index(sections, idx, codec):
         cncx_records = [x[0] for x in sections[off:off+indx_header['ncncx']]]
         cncx = CNCX(cncx_records, codec)
 
-    tag_section_start = indx_header['tagx']
+    tag_section_start = get_tag_section_start(data, indx_header)
     control_byte_count, tags = parse_tagx_section(data[tag_section_start:])
 
     for i in range(idx + 1, idx + 1 + indx_count):
