@@ -3,10 +3,8 @@
 
 import os
 from contextlib import suppress
-from qt.core import (
-    QBrush, QDialog, QDialogButtonBox, Qt, QTextCursor,
-    QTextEdit, pyqtSignal
-)
+
+from qt.core import QBrush, QDialog, QDialogButtonBox, Qt, QTextCursor, QTextEdit, pyqtSignal
 
 from calibre.constants import iswindows
 from calibre.ebooks.conversion.search_replace import compile_regular_expression
@@ -14,6 +12,7 @@ from calibre.gui2 import choose_files, error_dialog, gprefs
 from calibre.gui2.convert.regex_builder_ui import Ui_RegexBuilder
 from calibre.gui2.convert.xpath_wizard import XPathEdit
 from calibre.gui2.dialogs.choose_format import ChooseFormatDialog
+from calibre.gui2.widgets2 import to_plain_text
 from calibre.ptempfile import TemporaryFile
 from calibre.utils.icu import utf16_length
 from calibre.utils.ipc.simple_worker import WorkerError, fork_job
@@ -44,8 +43,7 @@ class RegexBuilder(QDialog, Ui_RegexBuilder):
         self.regex.textChanged[native_string_type].connect(self.regex_valid)
         for src, slot in (('test', 'do'), ('previous', 'goto'), ('next',
             'goto')):
-            getattr(self, src).clicked.connect(getattr(self, '%s_%s'%(slot,
-                src)))
+            getattr(self, src).clicked.connect(getattr(self, f'{slot}_{src}'))
         self.test.setDefault(True)
 
         self.match_locs = []
@@ -84,7 +82,7 @@ class RegexBuilder(QDialog, Ui_RegexBuilder):
             qt: int = 0
 
         if self.regex_valid():
-            text = str(self.preview.toPlainText())
+            text = to_plain_text(self.preview)
             regex = str(self.regex.text())
             cursor = QTextCursor(self.preview.document())
             extsel = QTextEdit.ExtraSelection()
@@ -190,7 +188,7 @@ class RegexBuilder(QDialog, Ui_RegexBuilder):
                     (pathtoebook, tf))
             except WorkerError as e:
                 return error_dialog(self, _('Failed to generate preview'),
-                        err_msg, det_msg=e.orig_tb, show=True)
+                        err_msg, det_msg=str(e) + '\n\n' + e.orig_tb, show=True)
             except:
                 import traceback
                 return error_dialog(self, _('Failed to generate preview'),
@@ -205,7 +203,7 @@ class RegexBuilder(QDialog, Ui_RegexBuilder):
             self.open_book(files[0])
 
     def doc(self):
-        return str(self.preview.toPlainText())
+        return to_plain_text(self.preview)
 
 
 class RegexEdit(XPathEdit):
