@@ -10,16 +10,15 @@ import re
 import traceback
 
 from calibre import prints, sanitize_file_name, strftime
-from calibre.constants import DEBUG, preferred_encoding
+from calibre.constants import DEBUG, iswindows, preferred_encoding
 from calibre.db.errors import NoSuchFormat
 from calibre.db.lazy import FormatsList
 from calibre.ebooks.metadata import fmt_sidx, title_sort
 from calibre.utils.config import Config, StringConfig, tweaks
 from calibre.utils.date import as_local_time, is_date_undefined
-from calibre.utils.filenames import (
-    ascii_filename, make_long_path_useable, shorten_components_to,
-)
+from calibre.utils.filenames import ascii_filename, make_long_path_useable, shorten_components_to
 from calibre.utils.formatter import TemplateFormatter
+from calibre.utils.formatter_functions import load_user_template_functions
 from calibre.utils.localization import _
 
 plugboard_any_device_value = 'any device'
@@ -30,26 +29,26 @@ plugboard_save_to_disk_value = 'save_to_disk'
 DEFAULT_TEMPLATE = '{author_sort}/{title}/{title} - {authors}'
 DEFAULT_SEND_TEMPLATE = '{author_sort}/{title} - {authors}'
 
-FORMAT_ARG_DESCS = dict(
-        title=_('The title'),
-        authors=_('The authors'),
-        author_sort=_('The author sort string. To use only the first letter '
+FORMAT_ARG_DESCS = {
+        'title': _('The title'),
+        'authors': _('The authors'),
+        'author_sort': _('The author sort string. To use only the first letter '
             'of the name use {author_sort[0]}'),
-        tags=_('The tags'),
-        series=_('The series'),
-        series_index=_('The series number. '
+        'tags': _('The tags'),
+        'series': _('The series'),
+        'series_index': _('The series number. '
             'To get leading zeros use {series_index:0>3s} or '
             '{series_index:>3s} for leading spaces'),
-        rating=_('The rating'),
-        isbn=_('The ISBN'),
-        publisher=_('The publisher'),
-        timestamp=_('The date'),
-        pubdate=_('The published date'),
-        last_modified=_('The date when the metadata for this book record'
+        'rating': _('The rating'),
+        'isbn': _('The ISBN'),
+        'publisher': _('The publisher'),
+        'timestamp': _('The date'),
+        'pubdate': _('The published date'),
+        'last_modified': _('The date when the metadata for this book record'
             ' was last modified'),
-        languages=_('The language(s) of this book'),
-        id=_('The calibre internal id')
-        )
+        'languages': _('The language(s) of this book'),
+        'id': _('The calibre internal id')
+}
 
 FORMAT_ARGS = {}
 for x in FORMAT_ARG_DESCS:
@@ -146,7 +145,7 @@ class Formatter(TemplateFormatter):
     '''
 
     def get_value(self, key, args, kwargs):
-        if key == '':
+        if not isinstance(key, str) or key == '':
             return ''
         try:
             key = key.lower()
@@ -173,10 +172,7 @@ class Formatter(TemplateFormatter):
             return key
 
 
-def get_components(template, mi, id, timefmt='%b %Y', length=250,
-        sanitize_func=ascii_filename, replace_whitespace=False,
-        to_lowercase=False, safe_format=True, last_has_extension=True,
-        single_dir=False):
+def get_component_metadata(template, mi, book_id, timefmt='%b %Y'):
     tsorder = tweaks['save_template_title_series_sorting']
     format_args = FORMAT_ARGS.copy()
     format_args.update(mi.all_non_none_fields())
@@ -202,8 +198,6 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
         format_args['series'] = title_sort(mi.series, order=tsorder)
         if mi.series_index is not None:
             format_args['series_index'] = mi.format_series_index()
-    else:
-        template = re.sub(r'\{series_index[^}]*?\}', '', template)
     if mi.rating is not None:
         format_args['rating'] = mi.format_rating(divide_by=2.0)
     if mi.identifiers:
@@ -215,11 +209,13 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
         format_args['timestamp'] = strftime(timefmt, mi.timestamp.timetuple())
     if not is_date_undefined(mi.pubdate) and hasattr(mi.pubdate, 'timetuple'):
         format_args['pubdate'] = strftime(timefmt, mi.pubdate.timetuple())
+    else:
+        format_args.pop('pubdate', None)
     if (hasattr(mi, 'last_modified') and not is_date_undefined(mi.last_modified) and
                 hasattr(mi.last_modified, 'timetuple')):
         format_args['last_modified'] = strftime(timefmt, mi.last_modified.timetuple())
 
-    format_args['id'] = str(id)
+    format_args['id'] = str(book_id)
     # Now format the custom fields
     custom_metadata = mi.get_all_user_metadata(make_copy=False)
     for key in custom_metadata:
@@ -241,15 +237,22 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
                     format_args[key] = str(format_args[key])
                 else:
                     format_args[key] = ''
+    return format_args
+
+
+def get_components(template, mi, book_id, timefmt='%b %Y', length=250,
+        sanitize_func=ascii_filename, replace_whitespace=False,
+        to_lowercase=False, safe_format=True, last_has_extension=True,
+        single_dir=False):
+    format_args = get_component_metadata(template, mi, book_id, timefmt)
     if safe_format:
-        components = Formatter().safe_format(template, format_args,
-                                            'G_C-EXCEPTION!', mi)
+        components = Formatter().safe_format(template, format_args, 'G_C-EXCEPTION!', mi)
     else:
         components = Formatter().unsafe_format(template, format_args, mi)
     components = [x.strip() for x in components.split('/')]
     components = [sanitize_func(x) for x in components if x]
     if not components:
-        components = [str(id)]
+        components = [str(book_id)]
     if to_lowercase:
         components = [x.lower() for x in components]
     if replace_whitespace:
@@ -342,13 +345,13 @@ def do_save_book_to_disk(db, book_id, mi, plugboards,
             cdata = db.cover(book_id)
             if cdata:
                 cpath = base_path + '.jpg'
-                with open(cpath, 'wb') as f:
+                with open(make_long_path_useable(cpath), 'wb') as f:
                     f.write(cdata)
                 mi.cover = base_name+'.jpg'
         if opts.write_opf:
             from calibre.ebooks.metadata.opf2 import metadata_to_opf
             opf = metadata_to_opf(mi)
-            with open(base_path+'.opf', 'wb') as f:
+            with open(make_long_path_useable(base_path+'.opf'), 'wb') as f:
                 f.write(opf)
     finally:
         mi.cover, mi.pubdate, mi.timestamp = originals
@@ -372,7 +375,7 @@ def do_save_book_to_disk(db, book_id, mi, plugboards,
         except NoSuchFormat:
             continue
         if opts.update_metadata:
-            with open(fmt_path, 'r+b') as stream:
+            with open(make_long_path_useable(fmt_path), 'r+b') as stream:
                 update_metadata(mi, fmt, stream, plugboards, cdata)
 
     return not formats_written, book_id, mi.title
@@ -384,10 +387,11 @@ def sanitize_args(root, opts):
     root = os.path.abspath(root)
 
     opts.template = preprocess_template(opts.template)
-    length = 240
+    length = 255 if iswindows else 1024
+    length -= 15
     length -= len(root)
     if length < 5:
-        raise ValueError('%r is too long.'%root)
+        raise ValueError(f'{root!r} is too long.')
     return root, opts, length
 
 
@@ -438,8 +442,10 @@ def read_serialized_metadata(data):
 
 
 def update_serialized_metadata(book, common_data=None):
+    # This is called from a worker process. It must not open the database.
     result = []
-    plugboard_cache = common_data
+    plugboard_cache = common_data['plugboard_cache']
+    load_user_template_functions(common_data['library_id'], common_data['template_functions'])
     from calibre.customize.ui import apply_null_metadata
     with apply_null_metadata:
         fmts = [fp.rpartition(os.extsep)[-1] for fp in book['fmts']]
