@@ -7,7 +7,10 @@ __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, string, traceback, numbers
+import numbers
+import re
+import string
+import traceback
 from collections import OrderedDict
 from functools import partial
 from math import modf
@@ -17,9 +20,9 @@ from calibre import prints
 from calibre.constants import DEBUG
 from calibre.ebooks.metadata.book.base import field_metadata
 from calibre.utils.config import tweaks
-from calibre.utils.formatter_functions import (
-    formatter_functions, get_database, function_object_type, StoredObjectType)
+from calibre.utils.formatter_functions import StoredObjectType, formatter_functions, function_object_type, get_database
 from calibre.utils.icu import strcmp
+from calibre.utils.localization import _
 from polyglot.builtins import error_message
 
 
@@ -55,6 +58,8 @@ class Node:
     NODE_LOCAL_FUNCTION_CALL = 29
     NODE_RANGE = 30
     NODE_SWITCH = 31
+    NODE_SWITCH_IF = 32
+    NODE_LIST_COUNT_FIELD = 33
 
     def __init__(self, line_number, name):
         self.my_line_number = line_number
@@ -187,7 +192,7 @@ class SetGlobalsNode(Node):
 
 class StringCompareNode(Node):
     def __init__(self, line_number, operator, left, right):
-        Node.__init__(self, line_number, 'comparision: ' + operator)
+        Node.__init__(self, line_number, 'comparison: ' + operator)
         self.node_type = self.NODE_COMPARE_STRING
         self.operator = operator
         self.left = left
@@ -289,6 +294,13 @@ class SwitchNode(Node):
         self.expression_list = expression_list
 
 
+class SwitchIfNode(Node):
+    def __init__(self, line_number, expression_list):
+        Node.__init__(self, line_number, 'switch_if()')
+        self.node_type = self.NODE_SWITCH_IF
+        self.expression_list = expression_list
+
+
 class ContainsNode(Node):
     def __init__(self, line_number, arguments):
         Node.__init__(self, line_number, 'contains()')
@@ -320,6 +332,13 @@ class StrcatNode(Node):
         self.expression_list = expression_list
 
 
+class ListCountFieldNode(Node):
+    def __init__(self, line_number, expression):
+        Node.__init__(self, line_number, 'list_count_field()')
+        self.node_type = self.NODE_LIST_COUNT_FIELD
+        self.expression = expression
+
+
 class _Parser:
     LEX_OP = 1
     LEX_ID = 2
@@ -557,8 +576,8 @@ class _Parser:
         while not self.token_op_is(')'):
             a = self.top_expr()
             if a.node_type not in (Node.NODE_ASSIGN, Node.NODE_RVALUE):
-                self.error(_("Parameters to a function must be "
-                             "variables or assignments"))
+                self.error(_('Parameters to a function must be '
+                             'variables or assignments'))
             if a.node_type == Node.NODE_RVALUE:
                 a = AssignNode(line_number, a.name, ConstantNode(self.line_number, ''))
             arguments.append(a)
@@ -633,7 +652,7 @@ class _Parser:
     def compare_expr(self):
         left = self.add_subtract_expr()
         if (self.token_op_is_string_infix_compare() or
-                self.token_is('in') or self.token_is('inlist')):
+                self.token_is('in') or self.token_is('inlist') or self.token_is('inlist_field')):
             operator = self.token()
             return StringCompareNode(self.line_number, operator, left, self.add_subtract_expr())
         if self.token_op_is_numeric_infix_compare():
@@ -667,8 +686,8 @@ class _Parser:
         return self.expr()
 
     keyword_nodes = {
-            'if':       (lambda self:None, if_expression),
-            'for':      (lambda self:None, for_expression),
+            'if':       (lambda self: None, if_expression),
+            'for':      (lambda self: None, for_expression),
             'break':    (lambda self: self.consume(), lambda self: BreakNode(self.line_number)),
             'continue': (lambda self: self.consume(), lambda self: ContinueNode(self.line_number)),
             'return':   (lambda self: self.consume(), lambda self: ReturnNode(self.line_number, self.top_expr())),
@@ -687,6 +706,8 @@ class _Parser:
                              lambda ln, args: FirstNonEmptyNode(ln, args)),
         'switch':           (lambda args: len(args) >= 3 and (len(args) %2) == 0,
                              lambda ln, args: SwitchNode(ln, args)),
+        'switch_if':        (lambda args: len(args) > 0 and (len(args) %2) == 1,
+                             lambda ln, args: SwitchIfNode(ln, args)),
         'assign':           (lambda args: len(args) == 2 and len(args[0]) == 1 and args[0][0].node_type == Node.NODE_RVALUE,
                              lambda ln, args: AssignNode(ln, args[0][0].name, args[1])),
         'contains':         (lambda args: len(args) == 4,
@@ -696,7 +717,9 @@ class _Parser:
         'print':            (lambda _: True,
                              lambda ln, args: PrintNode(ln, args)),
         'strcat':           (lambda _: True,
-                             lambda ln, args: StrcatNode(ln, args))
+                             lambda ln, args: StrcatNode(ln, args)),
+        'list_count_field': (lambda args: len(args) == 1,
+                             lambda ln, args: ListCountFieldNode(ln, args[0]))
     }
 
     def expr(self):
@@ -746,7 +769,7 @@ class _Parser:
 
             # Eat the opening paren, parse the argument list, then eat the closing paren
             self.consume()
-            arguments = list()
+            arguments = []
             while not self.token_op_is(')'):
                 # parse an argument expression (recursive call)
                 arguments.append(self.expression_list())
@@ -830,7 +853,7 @@ class StopException(Exception):
         super().__init__('Template evaluation stopped')
 
 
-class PythonTemplateContext(object):
+class PythonTemplateContext:
 
     def __init__(self):
         # Set attributes we already know must exist.
@@ -838,7 +861,9 @@ class PythonTemplateContext(object):
         self.db = None
         self.arguments = None
         self.globals = None
-        self.attrs_set = {'db', 'arguments', 'globals'}
+        self.formatter = None
+        self.funcs = None
+        self.attrs_set = {'db', 'arguments', 'globals', 'funcs'}
 
     def set_values(self, **kwargs):
         # Create/set attributes from the named parameters. Doing it this way we
@@ -853,17 +878,99 @@ class PythonTemplateContext(object):
     @property
     def attributes(self):
         # return a list of attributes in the context object
-        return sorted(list(self.attrs_set))
+        return sorted(self.attrs_set)
 
     def __str__(self):
         # return a string of the attribute with values separated by newlines
-        attrs = sorted(list(self.attrs_set))
+        attrs = sorted(self.attrs_set)
         ans = OrderedDict()
         for k in attrs:
             ans[k] = getattr(self, k, None)
         return '\n'.join(f'{k}:{v}' for k,v in ans.items())
 
 
+class FormatterFuncsCaller:
+    '''
+    Provides a convenient solution to call functions loaded in the
+    TemplateFormatter. The functions are called using their name as an attribute
+    of this class, with an underscore at the end if the name conflicts with a
+    Python keyword. If the name contain a illegal character for a attribute
+    (like .:-), use getattr(). Example: context.funcs.list_re_group()
+    '''
+
+    def __init__(self, formatter):
+        if not isinstance(formatter, TemplateFormatter):
+            raise TypeError(f'{formatter} is not an instance of TemplateFormatter')
+        self.__formatter__ = formatter
+
+    def __getattribute__(self, name):
+        if name.startswith('__') and name.endswith('__'):  # return internal special attribute
+            try:
+                return object.__getattribute__(self, name)
+            except Exception:
+                pass
+
+        formatter = self.__formatter__
+        func_name = ''
+        if name.endswith('_') and name[:-1] in formatter.funcs:  # give the priority to the backup name
+            func_name = name[:-1]
+        elif name in formatter.funcs:
+            func_name = name
+
+        if func_name:
+
+            def call(*args, **kargs):
+                def n(d):
+                    return '' if d is None else str(d)
+                args = tuple(n(a) for a in args)
+
+                try:
+                    if kargs:
+                        raise ValueError(_('Keyword arguments are not allowed'))
+
+                    # special function
+                    if func_name == 'arguments':
+                        raise ValueError(_("Don't call {0}. Instead use {1}").format('arguments()', 'context.arguments'))
+                    if func_name == 'globals':
+                        raise ValueError(_("Don't call {0}. Instead use {1}").format('globals()', 'context.globals'))
+                    if func_name == 'set_globals':
+                        raise ValueError(_("Don't call {0}. Instead use {1}").format('set_globals()', "context.globals['name'] = val"))
+                    if func_name == 'character':
+                        if _Parser.inlined_function_nodes['character'][0](args):
+                            rslt = _Interpreter.characters.get(args[0])
+                            if rslt is None:
+                                raise ValueError(_("Invalid character name '{0}'").format(args[0]))
+                        else:
+                            raise ValueError(_('Incorrect number of arguments'))
+                    else:
+                        # built-in/user template functions and Stored GPM/Python templates
+                        func = formatter.funcs[func_name]
+                        if func.object_type == StoredObjectType.PythonFunction:
+                            rslt = func.evaluate(formatter, formatter.kwargs, formatter.book, formatter.locals, *args)
+                        else:
+                            rslt = formatter._eval_sfm_call(func_name, args, formatter.global_vars)
+
+                except Exception as e:
+                    # Change the error message to return the name used in the template
+                    e = e.__class__(_('Error in function {0} :: {1}').format(
+                            name,
+                            re.sub(r'\w+\.evaluate\(\)\s*', '', str(e), 1)))  # remove UserFunction.evaluate() | Builtin*.evaluate()
+                    e.is_internal = True
+                    raise e
+                return rslt
+
+            return call
+
+        e = AttributeError(_('No function named {!r} exists').format(name))
+        e.is_internal = True
+        raise e
+
+    def __dir__(self):
+        return list(set(object.__dir__(self) +
+                        list(self.__formatter__.funcs.keys()) +
+                        [f+'_' for f in self.__formatter__.funcs.keys()]))
+
+
 class _Interpreter:
     def error(self, message, line_number):
         m = _('Interpreter: {0} - line number {1}').format(message, line_number)
@@ -876,7 +983,7 @@ class _Interpreter:
         self.parent_book = parent.book
         self.funcs = funcs
         self.locals = {'$':val}
-        self.local_functions = dict()
+        self.local_functions = {}
         self.override_line_number = None
         self.global_vars = global_vars if isinstance(global_vars, dict) else {}
         if break_reporter:
@@ -888,7 +995,7 @@ class _Interpreter:
         try:
             if is_call:
                 # prog is an instance of the function definition class
-                ret =  self.do_node_stored_template_call(StoredTemplateCallNode(1, prog.name, prog, None), args=args)
+                ret = self.do_node_stored_template_call(StoredTemplateCallNode(1, prog.name, prog, None), args=args)
             else:
                 ret = self.expression_list(prog)
         except ReturnExecuted as e:
@@ -967,20 +1074,20 @@ class _Interpreter:
             try:
                 start_val = int(self.float_deal_with_none(self.expr(prog.start_expr)))
             except ValueError:
-                self.error(_("{0}: {1} must be an integer").format('for', 'start'), line_number)
+                self.error(_('{0}: {1} must be an integer').format('for', 'start'), line_number)
             try:
                 stop_val = int(self.float_deal_with_none(self.expr(prog.stop_expr)))
             except ValueError:
-                self.error(_("{0}: {1} must be an integer").format('for', 'stop'), line_number)
+                self.error(_('{0}: {1} must be an integer').format('for', 'stop'), line_number)
             try:
                 step_val = int(self.float_deal_with_none(self.expr(prog.step_expr)))
             except ValueError:
-                self.error(_("{0}: {1} must be an integer").format('for', 'step'), line_number)
+                self.error(_('{0}: {1} must be an integer').format('for', 'step'), line_number)
             try:
                 limit_val = (1000 if prog.limit_expr is None else
                          int(self.float_deal_with_none(self.expr(prog.limit_expr))))
             except ValueError:
-                self.error(_("{0}: {1} must be an integer").format('for', 'limit'), line_number)
+                self.error(_('{0}: {1} must be an integer').format('for', 'limit'), line_number)
             var = prog.variable
             if (self.break_reporter):
                 self.break_reporter("'for': start value", str(start_val), line_number)
@@ -992,7 +1099,7 @@ class _Interpreter:
                 range_gen = range(start_val, stop_val, step_val)
                 if len(range_gen) > limit_val:
                     self.error(
-                        _("{0}: the range length ({1}) is larger than the limit ({2})").format(
+                        _('{0}: the range length ({1}) is larger than the limit ({2})').format(
                             'for', str(len(range_gen)), str(limit_val)), line_number)
                 for x in (str(x) for x in range_gen):
                     try:
@@ -1021,7 +1128,7 @@ class _Interpreter:
             self.error(_("Unknown identifier '{0}'").format(prog.name), prog.line_number)
 
     def do_node_func(self, prog):
-        args = list()
+        args = []
         for arg in prog.expression_list:
             # evaluate the expression (recursive call)
             args.append(self.expr(arg))
@@ -1080,12 +1187,12 @@ class _Interpreter:
             self.break_reporter(prog.node_name, _('before evaluating arguments'), prog.line_number)
         line_number, argument_list, block  = self.local_functions[prog.name].attributes_to_tuple()
         if len(prog.arguments) > len(argument_list):
-            self.error(_("Function {0}: argument count mismatch -- "
-                         "{1} given, at most {2} required").format(prog.name,
+            self.error(_('Function {0}: argument count mismatch -- '
+                         '{1} given, at most {2} required').format(prog.name,
                                                           len(prog.arguments),
                                                           len(argument_list)),
                        prog.line_number)
-        new_locals = dict()
+        new_locals = {}
         for i,arg in enumerate(argument_list):
             if len(prog.arguments) > i:
                 new_locals[arg.left] = self.expr(prog.arguments[i])
@@ -1147,10 +1254,12 @@ class _Interpreter:
                 if (self.break_reporter):
                     self.break_reporter(prog.node_name, res, prog.line_number)
                 return res
+            except StopException:
+                raise
             except:
                 self.error(_("Unknown field '{0}'").format(name), prog.line_number)
-        except (StopException, ValueError) as e:
-            raise e
+        except (StopException, ValueError):
+            raise
         except:
             self.error(_("Unknown field '{0}'").format('internal parse error'),
                        prog.line_number)
@@ -1217,12 +1326,37 @@ class _Interpreter:
             self.break_reporter(prog.node_name, res, prog.line_number)
         return res
 
+    def do_node_switch_if(self, prog):
+        for i in range(0, len(prog.expression_list)-1, 2):
+            tst = self.expr(prog.expression_list[i])
+            if self.break_reporter:
+                self.break_reporter('switch_if(): test expr', tst, prog.line_number)
+            if tst:
+                res = self.expr(prog.expression_list[i+1])
+                if self.break_reporter:
+                    self.break_reporter('switch_if(): value expr', res, prog.line_number)
+                return res
+        res = self.expr(prog.expression_list[-1])
+        if (self.break_reporter):
+            self.break_reporter('switch_if(): default expr', res, prog.line_number)
+        return res
+
     def do_node_strcat(self, prog):
         res = ''.join([self.expr(expr) for expr in prog.expression_list])
         if self.break_reporter:
             self.break_reporter(prog.node_name, res, prog.line_number)
         return res
 
+    def do_node_list_count_field(self, prog):
+        name = field_metadata.search_term_to_field_key(self.expr(prog.expression))
+        res = getattr(self.parent_book, name, None)
+        if res is None or not isinstance(res, (list, tuple, set, dict)):
+            self.error(_("Field '{0}' is either not a field or not a list").format(name), prog.line_number)
+        ans = str(len(res))
+        if self.break_reporter:
+            self.break_reporter(prog.node_name, ans, prog.line_number)
+        return ans
+
     def do_node_break(self, prog):
         if (self.break_reporter):
             self.break_reporter(prog.node_name, '', prog.line_number)
@@ -1253,22 +1387,43 @@ class _Interpreter:
         return res
 
     INFIX_STRING_COMPARE_OPS = {
-        "==": lambda x, y: strcmp(x, y) == 0,
-        "!=": lambda x, y: strcmp(x, y) != 0,
-        "<": lambda x, y: strcmp(x, y) < 0,
-        "<=": lambda x, y: strcmp(x, y) <= 0,
-        ">": lambda x, y: strcmp(x, y) > 0,
-        ">=": lambda x, y: strcmp(x, y) >= 0,
-        "in": lambda x, y: re.search(x, y, flags=re.I),
-        "inlist": lambda x, y: list(filter(partial(re.search, x, flags=re.I),
+        '==': lambda x, y: strcmp(x, y) == 0,
+        '!=': lambda x, y: strcmp(x, y) != 0,
+        '<': lambda x, y: strcmp(x, y) < 0,
+        '<=': lambda x, y: strcmp(x, y) <= 0,
+        '>': lambda x, y: strcmp(x, y) > 0,
+        '>=': lambda x, y: strcmp(x, y) >= 0,
+        'in': lambda x, y: re.search(x, y, flags=re.I),
+        'inlist': lambda x, y: list(filter(partial(re.search, x, flags=re.I),
                                            [v.strip() for v in y.split(',') if v.strip()]))
         }
 
+    def do_inlist_field(self, left, right, prog):
+        res = getattr(self.parent_book, right, None)
+        if res is None or not isinstance(res, (list, tuple, set, dict)):
+            self.error(_("Field '{0}' is either not a field or not a list").format(right), prog.line_number)
+        pat = re.compile(left, flags=re.I)
+        if isinstance(res, dict):  # identifiers
+            for k,v in res.items():
+                if re.search(pat, f'{k}:{v}'):
+                    return '1'
+        else:
+            for v in res:
+                if re.search(pat, v):
+                    return '1'
+        return ''
+
     def do_node_string_infix(self, prog):
         try:
             left = self.expr(prog.left)
             right = self.expr(prog.right)
-            res = '1' if self.INFIX_STRING_COMPARE_OPS[prog.operator](left, right) else ''
+            try:
+                res = '1' if self.INFIX_STRING_COMPARE_OPS[prog.operator](left, right) else ''
+            except KeyError:
+                if prog.operator == 'inlist_field':
+                    res = self.do_inlist_field(left, right, prog)
+                else:
+                    raise
             if (self.break_reporter):
                 self.break_reporter(prog.node_name, res, prog.line_number)
             return res
@@ -1279,12 +1434,12 @@ class _Interpreter:
                          "operator '{0}'").format(prog.operator), prog.line_number)
 
     INFIX_NUMERIC_COMPARE_OPS = {
-        "==#": lambda x, y: x == y,
-        "!=#": lambda x, y: x != y,
-        "<#": lambda x, y: x < y,
-        "<=#": lambda x, y: x <= y,
-        ">#": lambda x, y: x > y,
-        ">=#": lambda x, y: x >= y,
+        '==#': lambda x, y: x == y,
+        '!=#': lambda x, y: x != y,
+        '<#': lambda x, y: x < y,
+        '<=#': lambda x, y: x <= y,
+        '>#': lambda x, y: x > y,
+        '>=#': lambda x, y: x >= y,
         }
 
     def float_deal_with_none(self, v):
@@ -1433,6 +1588,7 @@ class _Interpreter:
         Node.NODE_CALL_STORED_TEMPLATE:  do_node_stored_template_call,
         Node.NODE_FIRST_NON_EMPTY:       do_node_first_non_empty,
         Node.NODE_SWITCH:                do_node_switch,
+        Node.NODE_SWITCH_IF:             do_node_switch_if,
         Node.NODE_FOR:                   do_node_for,
         Node.NODE_RANGE:                 do_node_range,
         Node.NODE_GLOBALS:               do_node_globals,
@@ -1451,6 +1607,7 @@ class _Interpreter:
         Node.NODE_BINARY_STRINGOP:       do_node_stringops,
         Node.NODE_LOCAL_FUNCTION_DEFINE: do_node_local_function_define,
         Node.NODE_LOCAL_FUNCTION_CALL:   do_node_local_function_call,
+        Node.NODE_LIST_COUNT_FIELD:      do_node_list_count_field,
         }
 
     def expr(self, prog):
@@ -1492,6 +1649,9 @@ class TemplateFormatter(string.Formatter):
         self._template_parser = None
         self.recursion_stack = []
         self.recursion_level = -1
+        self._caller = None
+        self.python_context_object = None
+        self.database = None
 
     def _do_format(self, val, fmt):
         if not fmt or not val:
@@ -1539,23 +1699,24 @@ class TemplateFormatter(string.Formatter):
     # ################# Template language lexical analyzer ######################
 
     lex_scanner = re.Scanner([
-            (r'(==#|!=#|<=#|<#|>=#|>#)', lambda x,t: (_Parser.LEX_NUMERIC_INFIX, t)),  # noqa
-            (r'(==|!=|<=|<|>=|>)',       lambda x,t: (_Parser.LEX_STRING_INFIX, t)),  # noqa
-            (r'(if|then|else|elif|fi)\b',lambda x,t: (_Parser.LEX_KEYWORD, t)),  # noqa
-            (r'(for|in|rof|separator)\b',lambda x,t: (_Parser.LEX_KEYWORD, t)),  # noqa
-            (r'(separator|limit)\b',     lambda x,t: (_Parser.LEX_KEYWORD, t)),  # noqa
-            (r'(def|fed|continue)\b',    lambda x,t: (_Parser.LEX_KEYWORD, t)),  # noqa
-            (r'(return|inlist|break)\b', lambda x,t: (_Parser.LEX_KEYWORD, t)),  # noqa
-            (r'(\|\||&&|!|{|})',         lambda x,t: (_Parser.LEX_OP, t)),  # noqa
-            (r'[(),=;:\+\-*/&]',         lambda x,t: (_Parser.LEX_OP, t)),  # noqa
-            (r'-?[\d\.]+',               lambda x,t: (_Parser.LEX_CONST, t)),  # noqa
-            (r'\$\$?#?\w+',              lambda x,t: (_Parser.LEX_ID, t)),  # noqa
-            (r'\$',                      lambda x,t: (_Parser.LEX_ID, t)),  # noqa
-            (r'\w+',                     lambda x,t: (_Parser.LEX_ID, t)),  # noqa
-            (r'".*?((?<!\\)")',          lambda x,t: (_Parser.LEX_CONST, t[1:-1])),  # noqa
-            (r'\'.*?((?<!\\)\')',        lambda x,t: (_Parser.LEX_CONST, t[1:-1])),  # noqa
-            (r'\n#.*?(?:(?=\n)|$)',      lambda x,t: _Parser.LEX_NEWLINE),  # noqa
-            (r'\s',                      lambda x,t: _Parser.LEX_NEWLINE if t == '\n' else None),  # noqa
+            (r'(==#|!=#|<=#|<#|>=#|>#)', lambda x,t: (_Parser.LEX_NUMERIC_INFIX, t)),
+            (r'(==|!=|<=|<|>=|>)',       lambda x,t: (_Parser.LEX_STRING_INFIX, t)),
+            (r'(if|then|else|elif|fi)\b',lambda x,t: (_Parser.LEX_KEYWORD, t)),
+            (r'(for|in|rof|separator)\b',lambda x,t: (_Parser.LEX_KEYWORD, t)),
+            (r'(separator|limit)\b',     lambda x,t: (_Parser.LEX_KEYWORD, t)),
+            (r'(def|fed|continue)\b',    lambda x,t: (_Parser.LEX_KEYWORD, t)),
+            (r'(return|inlist|break)\b', lambda x,t: (_Parser.LEX_KEYWORD, t)),
+            (r'(inlist_field)\b',        lambda x,t: (_Parser.LEX_KEYWORD, t)),
+            (r'(\|\||&&|!|{|})',         lambda x,t: (_Parser.LEX_OP, t)),
+            (r'[(),=;:\+\-*/&]',         lambda x,t: (_Parser.LEX_OP, t)),
+            (r'-?[\d\.]+',               lambda x,t: (_Parser.LEX_CONST, t)),
+            (r'\$\$?#?\w+',              lambda x,t: (_Parser.LEX_ID, t)),
+            (r'\$',                      lambda x,t: (_Parser.LEX_ID, t)),
+            (r'\w+',                     lambda x,t: (_Parser.LEX_ID, t)),
+            (r'".*?((?<!\\)")',          lambda x,t: (_Parser.LEX_CONST, t[1:-1])),
+            (r'\'.*?((?<!\\)\')',        lambda x,t: (_Parser.LEX_CONST, t[1:-1])),
+            (r'\n#.*?(?:(?=\n)|$)',      lambda x,t: _Parser.LEX_NEWLINE),
+            (r'\s',                      lambda x,t: _Parser.LEX_NEWLINE if t == '\n' else None),
         ], flags=re.DOTALL)
 
     def _eval_program(self, val, prog, column_name, global_vars, break_reporter):
@@ -1598,13 +1759,28 @@ class TemplateFormatter(string.Formatter):
 
     def _run_python_template(self, compiled_template, arguments):
         try:
+            db = get_database(self.book, None)
+            db = db if db is not None else self.database
             self.python_context_object.set_values(
-                         db=get_database(self.book, get_database(self.book, None)),
+                         db=db,
                          globals=self.global_vars,
-                         arguments=arguments)
+                         arguments=arguments,
+                         formatter=self,
+                         funcs=self._caller)
             rslt = compiled_template(self.book, self.python_context_object)
+        except StopException:
+            raise
         except Exception as e:
-            ss = traceback.extract_tb(exc_info()[2])[-1]
+            stack = traceback.extract_tb(exc_info()[2])
+            ss = stack[-1]
+            if getattr(e, 'is_internal', False):
+                # Exception raised by FormatterFuncsCaller
+                # get the line inside the current template instead of the FormatterFuncsCaller
+                for s in reversed(stack):
+                    if s.filename == '<string>':
+                        ss = s
+                        break
+
             raise ValueError(_('Error in function {0} on line {1} : {2} - {3}').format(
                             ss.name, ss.lineno, type(e).__name__, str(e)))
         if not isinstance(rslt, str):
@@ -1628,7 +1804,7 @@ class TemplateFormatter(string.Formatter):
             raise ValueError(
                 _('Syntax error on line {0} column {1}: text {2}').format(e.lineno, e.offset, e.text))
         except KeyError:
-            raise ValueError(_("The {0} function is not defined in the template").format('evaluate'))
+            raise ValueError(_('The {0} function is not defined in the template').format('evaluate'))
 
     # ################# Override parent classes methods #####################
 
@@ -1647,13 +1823,13 @@ class TemplateFormatter(string.Formatter):
 
         # Handle functions
         # First see if we have a functional-style expression
-        if fmt.startswith('\''):
+        if fmt.startswith("'"):
             p = 0
         else:
-            p = fmt.find(':\'')
+            p = fmt.find(":'")
             if p >= 0:
                 p += 1
-        if p >= 0 and fmt[-1] == '\'':
+        if p >= 0 and fmt[-1] == "'":
             val = self._eval_program(val, fmt[p+1:-1], None, self.global_vars, None)
             colon = fmt[0:p].find(':')
             if colon < 0:
@@ -1726,27 +1902,12 @@ class TemplateFormatter(string.Formatter):
     # reference can use different parameters when calling safe_format(). Because
     # the parameters are saved as instance variables they can possibly affect
     # the 'calling' template. To avoid this problem, save the current formatter
-    # state when recursion is detected. There is no point in saving the level
-    # 0 state.
+    # state when recursion is detected. Save state at level zero to be sure that
+    # all class instance variables are restored to their base settings.
 
     def save_state(self):
         self.recursion_level += 1
-        if self.recursion_level > 0:
-            return (
-                (self.strip_results,
-                 self.column_name,
-                 self.template_cache,
-                 self.kwargs,
-                 self.book,
-                 self.global_vars,
-                 self.funcs,
-                 self.locals))
-        else:
-            return None
-
-    def restore_state(self, state):
-        self.recursion_level -= 1
-        if state is not None:
+        return (
             (self.strip_results,
              self.column_name,
              self.template_cache,
@@ -1754,7 +1915,26 @@ class TemplateFormatter(string.Formatter):
              self.book,
              self.global_vars,
              self.funcs,
-             self.locals) = state
+             self.locals,
+             self._caller,
+             self.python_context_object,
+             self.database))
+
+    def restore_state(self, state):
+        self.recursion_level -= 1
+        if state is None:
+            raise ValueError(_('Formatter state restored before saved'))
+        (self.strip_results,
+         self.column_name,
+         self.template_cache,
+         self.kwargs,
+         self.book,
+         self.global_vars,
+         self.funcs,
+         self.locals,
+         self._caller,
+         self.python_context_object,
+         self.database) = state
 
     # Allocate an interpreter if the formatter encounters a GPM or TPM template.
     # We need to allocate additional interpreters if there is composite recursion
@@ -1783,6 +1963,7 @@ class TemplateFormatter(string.Formatter):
                       python_context_object=None):
         state = self.save_state()
         try:
+            self._caller = FormatterFuncsCaller(self)
             self.strip_results = strip_results
             self.column_name = self.template_cache = None
             self.kwargs = kwargs
@@ -1804,13 +1985,15 @@ class TemplateFormatter(string.Formatter):
                     column_name=None, template_cache=None,
                     strip_results=True, template_functions=None,
                     global_vars=None, break_reporter=None,
-                    python_context_object=None):
+                    python_context_object=None, database=None):
         state = self.save_state()
         if self.recursion_level == 0:
-            # Initialize the composite values dict if this is the base-level
-            # call. Recursive calls will use the same dict.
+            # Initialize the composite values dict and database if this is the
+            # base-level call. Recursive calls will use the same dict.
             self.composite_values = {}
+            self.database = database
         try:
+            self._caller = FormatterFuncsCaller(self)
             self.strip_results = strip_results
             self.column_name = column_name
             self.template_cache = template_cache
