@@ -4,13 +4,14 @@
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, sys
-from threading import Thread
+import os
+import sys
 from collections import namedtuple
 from multiprocessing.connection import Pipe
+from threading import Thread
 
-from calibre import detect_ncpus, as_unicode, prints
-from calibre.constants import iswindows, DEBUG
+from calibre import as_unicode, detect_ncpus, prints
+from calibre.constants import DEBUG, iswindows
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils import join_with_timeout
 from calibre.utils.ipc import eintr_retry_call
@@ -39,7 +40,7 @@ if iswindows:
     # Note that when running via the "Restart in debug mode" action, stdout is
     # not a console (its already redirected to a log file), so no redirection
     # is required.
-    if getattr(sys, 'gui_app', False) or getattr(sys.stdout, 'isatty', lambda : False)():
+    if getattr(sys, 'gui_app', False) or getattr(sys.stdout, 'isatty', lambda: False)():
         if DEBUG:
             # We are running in a windows console with calibre-debug -g
             import subprocess
@@ -352,7 +353,7 @@ def worker_main(conn):
             break
         if not isinstance(job, Job):
             if isinstance(job, File):
-                with lopen(job.name, 'rb') as f:
+                with open(job.name, 'rb') as f:
                     common_data = f.read()
                 common_data = pickle_loads(common_data)
             else:
@@ -456,7 +457,7 @@ def test():
     for r in itervalues(get_results(p)):
         c += 1
         if not r.traceback or 'ZeroDivisionError' not in r.traceback:
-            raise SystemExit('Unexpected result: %s' % r)
+            raise SystemExit(f'Unexpected result: {r}')
     if c != 1000:
         raise SystemExit('Incorrect number of results')
     p.shutdown(), p.join()
